# This script generates Figure 2, intraday quote frequency

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/analysis/");

# Preamble
freq = c(100,5*60); 
# freq[1] is the sampling frequency, expressed in milliseconds
# freq[2] is the subsample length, expressed in seconds
dates     = c(20150119:20150123,20150126:20150130);

# Loading and merging Olsen and ICAP data
load(file="input/dealer_quotes");
load(file="input/EBS_quotes");
nlab = c(brokers,"EBS");
quotes[[9]] = as.matrix(qq[,1:3])
N = length(nlab);
rm(qq);

# Intraday quoting activity
times = seq(0,24,0.5)*3600; # half hour intervals
time.labels = paste(rep(0:23,each=2), c(0,30), sep=":")
quote.count = matrix(0,ncol=length(nlab),nrow=length(times)-1,dimnames=list(time.labels,nlab))
for(b in 1:length(nlab)){
  qq = quotes[[b]];

  # Filter such that only price *changes* are retained
  if(b == N) qq = qq[c(1,qq[-1,"mid"]-qq[-nrow(qq),"mid"])!=0,]
  
  # Count of number of observations per half hour period intraday
  tmp = table(findInterval(qq[qq[,1] %in% dates,"time"],times))
  quote.count[as.numeric(names(tmp)),b] = tmp/length(dates);
}
# Adjusting to the Swiss time zone
quote.count = rbind(quote.count[nrow(quote.count)-1:0,],quote.count[1:(nrow(quote.count)-2),]);
dimnames(quote.count)[[1]] = time.labels;

# Plot Figure 2
pdf('output/Fig2.pdf')
plot(rowSums(quote.count,na.rm = TRUE), type="h", lwd = 6)
points(rowSums(quote.count[,-N],na.rm = TRUE), type="h", col= "blue", lwd = 6)
dev.off()
