# This script draws the network plots of Fig.3 and creates Table 1, Table 3B, and Table 6

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/analysis/");

# Loading required R packages
require(data.table);
require(dplyr);
require(igraph);
require(ggraph);
require(ggrepel);
require(ggforce);
require(extrafont);
font_import(paths = NULL, recursive = TRUE, prompt = TRUE, pattern = NULL)
extrafont::loadfonts(device="win")

# Loading bootstrap network stats for each time interval of the crash day (from script 1f)
load(file="input/crashNetworksSim");

significanceLevel = 0.05;
threshold.p = 1 - (1-significanceLevel)^(1/(N*(N-1)/2)) # 0.05;

mktOrdering = match(c("EBS","BBBB","DCFX","PEP0","FXN0","PFD0","DUBA","TFS0","WBRS"),nlab);
nlab2 = c("EBS",paste0("DLR",1:8));
nlab3 = c("EBS",paste0("DLR",match(nlab[mktOrdering][2:9],nlab)));

# Function for reporting mean network properties across parameter draws
meanNet = function(rows,cols){
  means = rep(NA,length(cols));
  pvals = rep(NA,length(cols));
  for(i in cols){
    if(var(crashNetworksSim[[s]][rows,i,with=F],na.rm=T)>0 & !is.na(var(crashNetworksSim[[s]][rows,i,with=F],na.rm=T))){
      tmp = t.test(crashNetworksSim[[s]][rows,i,with=F]);
      means[i-min(cols)+1] = tmp$estimate;
      pvals[i-min(cols)+1] = tmp$p.value;
    } else {
      means[i-min(cols)+1] = mean(unlist(crashNetworksSim[[s]][rows,i,with=F]),na.rm=T);
    }
  }
  out = c(means,pvals)
  return(out);
}


# Intraday intervals
subsetFileLabels = dimnames(subsetTimes)[[1]];
subsetLabels = c("11:30 AM -- 11:40 AM","11:45 AM -- 12:10 PM","12:10 PM -- 12:30 PM","12:30 PM -- 1:15 PM","1:15 PM -- 2:15 PM","2:15 PM -- 3:15 PM","3:15 PM -- 4:15 PM","4:15 PM -- 5:00 PM")
tau = 5;

# Table 6: Information shares (appendix)
Table6 = matrix(ncol=N, nrow=nrow(subsetTimes),dimnames=list(subsetLabels,nlab));
for(s in 1:nrow(subsetTimes)){
  Table6[s,match(vecm[[s]]$nlab, nlab)] = vecm[[s]]$ISl;
}
dimnames(Table6)[[2]][1:8] = paste0("DLR",1:8);
capture.output(round(100*t(Table6[,c(9,1:8)]),2), file = "output/Table6.txt")


# Network plots
AdjMatrixTable_crash = vector("list",length=nrow(subsetTimes))
for(s in 1:nrow(subsetTimes)){
  N.tmp = sum(grepl("PI.",dimnames(crashNetworksSim[[s]])[[2]]))
  nlab.tmp = substring(dimnames(crashNetworksSim[[s]])[[2]][grepl("PI.",dimnames(crashNetworksSim[[s]])[[2]])],4)
  
  # Adjacency matrix averaged across parameter draws
  subset = which(crashNetworksSim[[s]]$eventTime == tau);
  edges = aggregate(subset, by = list(crashNetworksSim[[s]]$eventTime[subset]), meanNet, cols=2+1:(2*(N^2)));
  dimnames(edges$x)[[2]] = paste0(dimnames(crashNetworksSim[[s]])[[2]][2+1:(2*(N^2))],rep(c(".m",".p"),each = 2*(N^2)))

  # Price inefficiency averaged across parameter draws
  distance = aggregate(subset, by = list(crashNetworksSim[[s]]$eventTime[subset]), meanNet, cols=(2+2*(N^2))+1:N.tmp);
  dimnames(distance$x)[[2]] = paste0(dimnames(crashNetworksSim[[s]])[[2]][(2+2*(N^2))+1:N.tmp],rep(c(".m",".p"),each = N.tmp))

  PI = vector("list",length=N.tmp);
  for(i in 1:N.tmp){
    PI[[i]] = data.table(eventTime = distance$Group.1, D = distance$x[,paste0("PI.",nlab.tmp[i],".m")],mkt=nlab.tmp[i])
  }
  PI = do.call(rbind,PI);
  PI$mkt2 = nlab3[match(PI$mkt,nlab[mktOrdering])];
  PI$label = NA;
  PI$label[PI$eventTime==0] = PI$mkt2[PI$eventTime==0];

  # Price inefficiency --> vertex coordinates
  PI.tau = rep(1,N);
  names(PI.tau) = nlab[mktOrdering];
  PI.tau[match(nlab.tmp,nlab[mktOrdering])] = PI$D[PI$eventTime==tau];
  tmp = max(c(PI.tau,1))+0.1;
  coordinates     = data.frame(x = c(tmp,-tmp,0,PI.tau * sin(2 * pi * ((0:(N - 1))/N))), y = c(tmp,-tmp,0,PI.tau * cos(2 * pi * ((0:(N - 1))/N))));
  
  # Adjacency matrix
  adjMatrix              = matrix(0,N,N, dimnames= list(nlab,nlab));
  adjMatrix.p            = matrix(0,N,N, dimnames= list(nlab,nlab));
  adjMatrix[nlab,nlab]   = matrix(edges$x[1:(N^2)],N,N);
  adjMatrix.p[nlab,nlab] = matrix(edges$x[2*(N^2)+1:(N^2)],N,N);
  
  adjMatrix      = rbind(matrix(0,3,12),cbind(matrix(0,9,3),adjMatrix[mktOrdering,mktOrdering]))
  adjMatrix.p    = rbind(matrix(0,3,12),cbind(matrix(0,9,3),adjMatrix.p[mktOrdering,mktOrdering]))
  
  # Clockwise numbering of dealers
  dimnames(adjMatrix)   = list(c("",""," ",nlab3),c("A","B"," ",nlab3))
  dimnames(adjMatrix.p) = list(c("",""," ",nlab3),c("A","B"," ",nlab3))
  
  AdjMatrixTable_crash[[s]] = list(adjMatrix = adjMatrix[nlab2,nlab2],p.values = round(adjMatrix.p[nlab2,nlab2],5),significance.thresholds = threshold.p);

  # Structuring data for plotting and 
  # determining what edges should be plotted (those that are statistically significant)
  cors = data.frame(x = dimnames(adjMatrix)[[2]][combn(1:(N+3),2)[1,]], y = dimnames(adjMatrix)[[2]][combn(1:(N+3),2)[2,]], r = as.vector(adjMatrix[lower.tri(adjMatrix)]), p = as.vector(adjMatrix.p[lower.tri(adjMatrix.p)]));
  graph_cors = graph_from_data_frame(as.tbl(cors), directed = FALSE);
  
  # Vertex label coordinates
  label.coord = data.frame(x = c(0,0,0,0.165 * sin(2 * pi * ((0:(N - 1))/N))), y = c(0,0,0,0.165 * cos(2 * pi * ((0:(N - 1))/N))));
  label.coord[3+1:N,][PI.tau>0.9,] = data.frame(x = c(0,0,0,-0.25 * sin(2 * pi * ((0:(N - 1))/N))), y = c(0,0,0,-0.25 * cos(2 * pi * ((0:(N - 1))/N))))[3+1:N,][PI.tau>0.9,];

  # Plotting
  netPlot = ggraph(graph_cors, layout = "manual", node.positions = coordinates) +
    geom_circle(aes(r=1,y0=0,x0=0), col = "red", linetype = "dashed") +
    geom_edge_link(aes(edge_alpha = abs(r), edge_width = abs(r), color = r)) +
    guides(edge_alpha = "none", edge_width = "none", color = "none") +
    scale_edge_colour_gradient2(limits = c(-1, 1), low = "firebrick2", mid = "white", high = "dodgerblue2", guide="none") +
    geom_node_point(fill = c("white","white","red",c("white","gray")[nlab[mktOrdering] %in% nlab.tmp+1]), size = c(0,0,3,c(0,5)[nlab[mktOrdering] %in% nlab.tmp+1]), shape = c(0,0,18,c(0,21)[nlab[mktOrdering] %in% nlab.tmp+1]), stroke = c(0,0,0,c(0,2)[(PI.tau<(-0.05))+1]), colour = c("white","white","red",c("black","yellow")[(PI.tau<(-0.05))+1])) +
    geom_node_text(aes(label = name), repel = F, color = c(rep("white",3),c("white","black")[nlab[mktOrdering] %in% nlab.tmp+1]), nudge_x = label.coord$x, nudge_y = label.coord$y) +
    theme_graph() +
    theme(plot.margin=grid::unit(c(0.1,0.1,0.1,0.1), "mm"))
  
  ggsave(paste0("output/Fig6_",subsetFileLabels[s],".pdf"), h=4,w=4, netPlot)
  #ggsave(paste0("output/CIRnetwork_",subsetFileLabels[s],"_pcor_",if(tau<1000) "0",if(tau<100) "0",if(tau<10) "0",tau,".pdf"), h=4,w=4, netPlot)
}

# Correlations of adjacency matrices intraday Jan 15 and baseline sample: CentralityAll
corMatrix  = matrix(ncol=2,nrow=nrow(subsetTimes),dimnames=list(dimnames(subsetTimes)[[1]],c("All edges", "Significant edges")));

load(file="temp/adjMatrix");

adjMatrix1 = as.vector(AdjMatrixTable[[1]]$adjMatrix[lower.tri(AdjMatrixTable[[1]]$adjMatrix)]);
adjMatrix2 = as.vector(AdjMatrixTable[[1]]$adjMatrix[lower.tri(AdjMatrixTable[[1]]$adjMatrix)]);
adjMatrix.p  = as.vector(AdjMatrixTable[[1]]$p.values[lower.tri(AdjMatrixTable[[1]]$adjMatrix)]);
adjMatrix1[adjMatrix1<0] = 0;
adjMatrix2[adjMatrix.p>threshold.p | adjMatrix2<0] = 0;
for(s in 1:nrow(subsetTimes)){
  adjMatrixCrash1 = as.vector(AdjMatrixTable_crash[[s]]$adjMatrix[lower.tri(AdjMatrixTable_crash[[s]]$adjMatrix)]);
  adjMatrixCrash2 = as.vector(AdjMatrixTable_crash[[s]]$adjMatrix[lower.tri(AdjMatrixTable_crash[[s]]$adjMatrix)]);
  adjMatrix.p  = as.vector(AdjMatrixTable_crash[[s]]$p.values[lower.tri(AdjMatrixTable_crash[[s]]$adjMatrix)]);
  adjMatrixCrash1[adjMatrixCrash1<0] = 0;
  adjMatrixCrash2[adjMatrix.p>threshold.p | adjMatrixCrash2<0] = 0;
  
  corMatrix[s,1] = cor(adjMatrix1,adjMatrixCrash1, use= "pairwise.complete.obs");
  corMatrix[s,2] = cor(adjMatrix2,adjMatrixCrash2, use= "pairwise.complete.obs");
}

round(corMatrix,2); # Section 5.5 states that the adj matrix correlation between normal times and the 16:15 crash network is 0.64
