# This script prepares the data on dealer quotes for subsequent analysis 

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/build/");

# Loading required R packages
require(data.table)
require(zoo)

# EUR/CHF data
load(file = "input/dealer_pseudodata")
setnames(raw,c("dateRaw","timeRaw","bid","ask","mkt","score"))

# The scores assigned by Olsen are >.9 in 97% of the cases. 
# Excluding quotes with lower scores
mean(raw$score<0.9); # 2.5%, reported in footnote 13
raw = raw[raw$score>0.9];

# Obtaining bid-ask spread midpoints
raw$mid = (raw$bid + raw$ask) / 2;

# Time conversion to seconds after midnight
# Date format conversion
time = strptime(raw$timeRaw,format="%H:%M:%OS")
raw$time = time$hour*3600 + time$min*60 + time$sec
date = strptime(raw$dateRaw,format="%d-%m-%Y")
raw$date = (1900+date$year)*10000 + (date$mon+1)*100 + date$mday
rm(time,date)

# Top-8 traders constitute 68% of OTC quote updates
dates = c(20150115,20150119:20150123,20150126:20150130);
brokers = c("FXN0", "DCFX", "TFS0", "DUBA", "PFD0", "WBRS", "BBBB", "PEP0")
subsample = raw$date %in% dates & raw$time > 3600*8 & raw$time < 3600*16;
sum(sort(table(raw$mkt[subsample & raw$mkt %in% brokers]),decreasing=TRUE))/sum(subsample); # 68%

quotes = vector("list",length=length(brokers));
for(b in 1:length(brokers)){
  quotes[[b]] = as.matrix(raw[raw$date %in% dates & raw$mkt==brokers[b],9:7,with=F]);
  dimnames(quotes[[b]])[[2]][1] = "date"
}
# Exporting quote midpoints 
save(list=c("quotes","brokers"),file="temp/dealer_quotes");

# Obtaining relative and nominal bid-ask spreads
raw$nomSpr = raw$ask - raw$bid;
raw$relSpr = 10000* (raw$nomSpr) / (raw$mid);
relSpr = vector("list",length=length(brokers));
nomSpr = vector("list",length=length(brokers));
for(b in 1:length(brokers)){
  relSpr[[b]] = as.matrix(raw[raw$date %in% dates & raw$mkt==brokers[b],c("date","time","relSpr"),with=F]);
  dimnames(relSpr[[b]])[[2]] = c("date","time","spr");
  nomSpr[[b]] = as.matrix(raw[raw$date %in% dates & raw$mkt==brokers[b],c("date","time","nomSpr"),with=F]);
  dimnames(nomSpr[[b]])[[2]] = c("date","time","spr");
}

# Exporting relative and nominal bid-ask spread
save(list=c("relSpr","nomSpr","brokers"),file="temp/dealer_spreads");

