# This script prepares the data on interdealer quotes and trades for subsequent analysis 

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/build/");

# Loading required R packages
require(data.table)
require(zoo)

# Date range
dates = c(20150115,20150119:20150123,20150126:20150130)

# EUR/CHF data from ICAP/EBS
EBSq = vector("list",length=length(dates))
for(d in 1:length(dates)){
  #raw = read.csv(gzfile(paste0("input/ICAP data/ehd_5p0-",dates[d],".ccy.out.gz")), as.is=TRUE, header=FALSE);
  load(file = "input/interdealer_pseudodata");
  qq = raw[raw[,4]=="Q" & raw[,3]=="EUR/CHF" & raw[,6]==1,-c(3,6)];
  dimnames(qq)[[2]] = c("rawdate","rawtime","type","side","price","vol","nq")

  # Time and date conversion
  time = strptime(qq$rawtime,format="%H:%M:%OS")
  date = strptime(qq$rawdate,format="%Y/%m/%d")
  qq$time = time$hour*3600 + time$min*60 + time$sec
  qq$date = (1900+date$year)*10000 + (date$mon+1)*100 + date$mday
  rm(time,date)
  
  # The bid-ask spread and its midpoint
  qq$bid = NA;
  qq$ask = NA;
  qq$bid[qq$side==0] = qq$price[qq$side==0];
  qq$ask[qq$side==1] = qq$price[qq$side==1];
  qq[,c("bid","ask")] = na.locf(qq[,c("bid","ask")],na.rm=F);
  qq$mid = (qq$bid + qq$ask)/2
  qq$nomSpr = qq$ask - qq$bid;
  qq$relSpr = 10000 * qq$nomSpr / qq$mid;
  qq = qq[qq$nomSpr >= 0 & qq$relSpr < 100 & !is.na(qq$relSpr),]

  # Saving the relevant data sets
  EBSq[[d]] = as.matrix(qq[,c("date","time","mid","nomSpr","relSpr")])
  dimnames(EBSq[[d]])[[2]][1] = "date";

  print(dates[d]);
}

qq = do.call(rbind, EBSq);
qq = qq[qq[,"date"] %in% dates & !is.na(qq[,"mid"]),]

# Saving
save(list=c("qq"),file="temp/EBS_quotes");
