# This script calculates the duration-weighted average quoted half-spread for each market and each subsample in the baseline sample

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/build/");

# Loading required R packages
require(data.table)
require(zoo)

# Preamble
freq      = c(100,5*60); 
# freq[1] is the sampling frequency expressed in milliseconds
# freq[2] is the length of the subsamples expressed in seconds
startTime = 8*3600;
endTime   = 16*3600;
N         = 9; # Number of markets
dates     = c(20150119:20150123,20150126:20150130);
rpd       = (endTime-startTime)/freq[2]; # subsamples per day

# Loading OTC spreads
load(file="temp/dealer_spreads");
brokers = c(brokers,"EBS")

# Adding EBS spreads
load(file="temp/EBS_quotes");
relSpr[[N]] = as.matrix(qq[,c("date","time","relSpr")]);
nomSpr[[N]] = as.matrix(qq[,c("date","time","nomSpr")]);
rm(qq);

# Calculating time-weighted bid-ask spreads
twSpreads = function(start,interval,mkt,date){
  rows = which(relSpr[[mkt]][,"time"] >= start & relSpr[[mkt]][,"time"] < (start+interval) & relSpr[[mkt]][,"date"] == date);
  if(length(rows)<3) return(c(date,start,mkt,NA,NA));
  rs   = cbind(relSpr[[mkt]][c(rows[1]-1,rows),],diff(c(start,relSpr[[mkt]][rows,"time"],start+interval)));
  ns   = cbind(nomSpr[[mkt]][c(rows[1]-1,rows),],diff(c(start,nomSpr[[mkt]][rows,"time"],start+interval)));
  return(c(date,start,mkt,weighted.mean(ns[,3],w=rs[,4]),weighted.mean(rs[,3],w=rs[,4])));
}
spreads_tw = t(mapply(twSpreads, date=rep(dates,each=rpd*N), 
                                 start = rep(seq(startTime,endTime-1,freq[2]),each=N),
                                 mkt = 1:N,
                                 interval = freq[2]));
spreads_tw = data.frame(date=spreads_tw[,1],time=spreads_tw[,2],mkt=brokers[spreads_tw[,3]],nomSpr=spreads_tw[,4],relSpr=spreads_tw[,5])
spreads_tw$subsample = match(paste0(spreads_tw$date,spreads_tw$time),unique(paste0(spreads_tw$date,spreads_tw$time)))

# Saving
save(list=c("spreads_tw","brokers"),file="temp/spreads_tw");
