# This script prepares the data on public news for subsequent analysis 

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/build/");

# Loading required R packages
require(data.table)
require(zoo)

freq      = c(100,5*60); 
# freq[1] is the sampling frequency expressed in milliseconds
# freq[2] is the length of the subsamples expressed in seconds
startTime = 8*3600;
endTime   = 16*3600;
N         = 9; # Number of markets
dates     = c(20150119:20150123,20150126:20150130);
rpd       = (endTime-startTime)/freq[2]; # subsamples per day
flash     = TRUE; # TRUE flags that only flash news stories considered

# Creating EBS price time series on second-by-second frequency
load(file="output/EBS_quotes");
qq = data.table(qq);
seconds = startTime:(endTime-1);
secondDates = data.table(cbind(as.data.frame(rep(unique(qq$date),each=length(seconds))),seconds))
dimnames(secondDates)[[2]] = c("date","time");
setkeyv(secondDates,c("date","time"))
setkeyv(qq,c("date","time"))
EBS.midpoints = qq[secondDates,roll=T];

# Calculating realized volatility in EBS prices for each subsample
EBStimes = seq(startTime, endTime-freq[2], freq[2])
EBStimes.labels = paste(trunc(EBStimes/3600,0),round(((EBStimes/3600)-trunc(EBStimes/3600,0))*60,0),sep=":")
EBS.midpoints$EBStimes = findInterval(EBS.midpoints$time,EBStimes)
RV = function(rows){
  mean((10000*diff(log(EBS.midpoints$mid[rows])))^2);
}
subset = EBS.midpoints$date %in% dates;
EBS.RV = aggregate((1:nrow(EBS.midpoints))[subset],by=list(EBS.midpoints$EBStimes[subset],EBS.midpoints$date[subset]),RV)
EBS.RV$time = EBStimes[EBS.RV$Group.1]
EBS.RV$timeLabel = EBStimes.labels[EBS.RV$Group.1]
EBS.RV = data.table(EBS.RV[,2:5])
dimnames(EBS.RV)[[2]][1:2] = c("date","RV")
setkeyv(EBS.RV,c("date","time"))

# Read and format news items
load(file = "input/news_pseudodata");
# Convert time stamps
time = strptime(substr(news$firstCreated,12,19),format="%H:%M:%OS")
date = strptime(substr(news$firstCreated,1,10),format="%Y-%m-%d")
news$time = time$hour*3600 + time$min*60 + time$sec
news$date = (1900+date$year)*10000 + (date$mon+1)*100 + date$mday

# News filtering
SNBnews = if(flash) data.table(news[news$bodySize == 0,c("date","time","headline")]) else data.table(news[,c("date","time","headline")]);
SNBnews$time[SNBnews$time %in% EBStimes] = SNBnews$time[SNBnews$time %in% EBStimes]-1; # -1 to avoid news story to match to two time periods
setkeyv(SNBnews,c("date","time"))

# Matching filtered news to EBS time stamps
EBStimesDates = data.table(cbind(as.data.frame(rep(dates,each=length(EBStimes))),EBStimes))
dimnames(EBStimesDates)[[2]] = c("date","time");
setkeyv(EBStimesDates,c("date","time"))
SNBnews.EBS = SNBnews[EBStimesDates,roll=-freq[2]]; 

save(list=c("EBS.RV","news","SNBnews.EBS"),file=paste0("temp/news_and_RV_",freq[2]/60,"min"));
