# This script assembles a panel of all measures required for the applications described in Sections 5.1 -- 5.4

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/build/");

# Loading required R packages
require(data.table);
require(igraph); # Used for assortativity measure

# Loading spreads (from script 1c)
load(file=paste0("temp/spreads_tw"));
timeLabel = data.frame(sec=seq(8,16-1/12,1/12)*3600,time=paste(rep(c("09",10:16),each=12),c("00","05",seq(10,55,5)),sep=":"))
subsampleStats = data.table(subsample = spreads_tw$subsample, date = spreads_tw$date, time = spreads_tw$time, timeLabel = timeLabel$time[match(round(spreads_tw$time,0),round(timeLabel$sec,0))], mkt = spreads_tw$mkt, relSpr = spreads_tw$relSpr)
subsampleStats$week = c("Week1","Week2")[findInterval(subsampleStats$date,c(20150119,20150126))];

# Adding VECM estimation stats, including information shares and network objects (from script 1e)
load(file="temp/VECM_100ms_5min_9markets_J10000K100");
#subsampleStats$L = NA;
subsampleStats$N = NA;
subsampleStats$conv = F;
subsampleStats$ISl = NA;
for(i in 1:nrow(nq1)){
  if(length(vecm[[i]]) > 1){
    #    subsampleStats$L[subset]    = vecm[[i]]$L;
    subsampleStats$N[subsampleStats$subsample==i]    = vecm[[i]]$N;
    subsampleStats$conv[subsampleStats$subsample==i] = sum(vecm[[i]]$conv>0.05) == 0;
    subsampleStats$ISl[subsampleStats$subsample==i & subsampleStats$mkt %in% vecm[[i]]$nlab] = vecm[[i]]$ISl;
  } 
}
eligible = sort(unique(subsampleStats$subsample[subsampleStats$conv])); # 11 subsamples not eligible

# Adding subsample information about news releases and realized volatility (from script 1d)
load(file="temp/news_and_RV_5min");
EBS.RV[, timeLabel:=NULL]
subsampleStats = merge(subsampleStats,EBS.RV,by=c("date","time"), all.x=T);

SNBnews.EBS$news = !is.na(SNBnews.EBS$headline);
subsampleStats = merge(subsampleStats,SNBnews.EBS[,c("date","time","news"),with=F],by=c("date","time"));
subsampleStats$news[!(subsampleStats$subsample %in% eligible)] = F;
length(unique(subsampleStats$subsample[subsampleStats$news])); # 68 news subsamples, as reported in Section 5.4

# Narrowing the news definition to top-quartile realized volatility subsamples with all markets active
newsPeriods = unique(subsampleStats$subsample[subsampleStats$news & subsampleStats$RV > quantile(subsampleStats$RV,0.75) & subsampleStats$N == N]);
length(newsPeriods)# 16 top-quartile news subsamples with 9 markets, as reported in Section 5.4

# Matching the news subsamples to no-news subsamples with matching volatility
subsampleStats$newsMatched = NA;
subsampleStats$newsMatched[subsampleStats$subsample %in% newsPeriods] = "News";

noNewsPeriods = rep(NA,length(newsPeriods));
for(t in 1:length(newsPeriods)){
  subset = !(subsampleStats$subsample %in% noNewsPeriods[!is.na(noNewsPeriods)]) & subsampleStats$subsample %in% eligible & subsampleStats$N==N & !(subsampleStats$subsample %in% newsPeriods) & subsampleStats$mkt=="EBS";
  noNewsPeriods[t] =  subsampleStats$subsample[subset][which.min(abs(subsampleStats$RV[subset] - subsampleStats$RV[subsampleStats$subsample == newsPeriods[t] & subsampleStats$mkt=="EBS"]))];
}
subsampleStats$newsMatched[subsampleStats$subsample %in% noNewsPeriods] = "NoNews";
rm(newsPeriods,noNewsPeriods,subset);
subsampleStats[,news:=NULL];

# Adding network properties for each subsample and each tau from zero to five
tauPeriods = 0:5; 
networkStats = vector("list",length=length(eligible)*length(tauPeriods))
for(w in 1:length(eligible)){
  i = eligible[w];

  # Obtaining the InfoSpeed measure
  speed = rep(NA,length(vecm[[i]]$nlab));
  for(j in 1:length(vecm[[i]]$nlab)){
    speed[j] = - as.numeric(rownames(vecm[[i]]$PI)[which(abs(vecm[[i]]$PI[,j])<0.2)[1]])
  }
  speed[is.na(speed)] = - 100;
  
  for(tau in 1:length(tauPeriods)){
    # To measure edge degree and  current and next-period changes in edge degree, the previous-tau, current-tau, and next-tau adjacency matrices are needed
    pcor.matrix.lag  = if(tauPeriods[tau] > 0) matrix(vecm[[i]]$pcorr[as.character(tauPeriods[tau]-1),],N,N, dimnames= list(nlab,nlab));
    pcor.matrix.lag  = if(tauPeriods[tau] > 0) pcor.matrix.lag[vecm[[i]]$nlab,vecm[[i]]$nlab];
    pcor.matrix      = matrix(vecm[[i]]$pcorr[as.character(tauPeriods[tau]),],N,N, dimnames= list(nlab,nlab));
    pcor.matrix      = pcor.matrix[vecm[[i]]$nlab,vecm[[i]]$nlab];
    pcor.matrix.lead = matrix(vecm[[i]]$pcorr[as.character(tauPeriods[tau]+1),],N,N, dimnames= list(nlab,nlab));
    pcor.matrix.lead = pcor.matrix.lead[vecm[[i]]$nlab,vecm[[i]]$nlab];
    
    # Obtaining changes in edge degree (dED) and price inefficiency (dPI) measures
    dED_0.1 = rowSums(pcor.matrix.lead > 0.1, na.rm=T) - rowSums(pcor.matrix > 0.1, na.rm=T);
    dED_0.2 = rowSums(pcor.matrix.lead > 0.2, na.rm=T) - rowSums(pcor.matrix > 0.2, na.rm=T);
    dED_all = (rowSums(pcor.matrix.lead * (pcor.matrix.lead > 0), na.rm=T) - rowSums(pcor.matrix * (pcor.matrix > 0), na.rm=T))/2;
    dPI     = vecm[[i]]$PI[as.character(tauPeriods[tau]+1),] - vecm[[i]]$PI[as.character(tauPeriods[tau]),];

    if(tauPeriods[tau] > 0){
      dED_0.1.lag = rowSums(pcor.matrix > 0.1, na.rm=T) - rowSums(pcor.matrix.lag > 0.1, na.rm=T);
      dED_0.2.lag = rowSums(pcor.matrix > 0.2, na.rm=T) - rowSums(pcor.matrix.lag > 0.2, na.rm=T);
      dED_all.lag = (rowSums(pcor.matrix * (pcor.matrix > 0),na.rm=T)    - rowSums(pcor.matrix.lag * (pcor.matrix.lag>0),na.rm=T))/2;
      dPI.lag     = vecm[[i]]$PI[as.character(tauPeriods[tau]),] - vecm[[i]]$PI[as.character(tauPeriods[tau]-1),];
    } else {
      dED_0.1.lag = NA;
      dED_0.2.lag = NA;
      dED_all.lag = NA;
      dPI.lag = NA;
    }
    
    # Obtaining the graph objects required for the assorativity coefficient, AssortCoef based on 0.1 and 0.2 thresholds
    # Using the graph.adjacency function from the igraph package
    pcor.matrix_0.1 = pcor.matrix;
    pcor.matrix_0.1[pcor.matrix_0.1 < 0.1] = 0;
    pcor.matrix_0.2 = pcor.matrix;
    pcor.matrix_0.2[pcor.matrix_0.1 < 0.2] = 0;

    g_0.1 <- graph.adjacency(pcor.matrix_0.1, mode="undirected", weighted=TRUE)
    g_0.2 <- graph.adjacency(pcor.matrix_0.2, mode="undirected", weighted=TRUE)

    # Output
    networkStats[[(w-1)*length(tauPeriods)+tau]] = data.frame(subsample = eligible[w],
                                                              mkt         = vecm[[i]]$nlab, 
                                                              speed       = speed, 
                                                              eventTime   = tauPeriods[tau],
                                                              PI          = vecm[[i]]$PI[as.character(tauPeriods[tau]),], 
                                                              ED_0.1      = rowSums(pcor.matrix > 0.1) - 1,
                                                              ED_0.2      = rowSums(pcor.matrix > 0.2) - 1,
                                                              ED_all      = (rowSums(pcor.matrix * (pcor.matrix > 0)) - 1) / 2,
                                                              dPI         = dPI, 
                                                              dED_0.1     = dED_0.1, 
                                                              dED_0.2     = dED_0.2, 
                                                              dED_all     = dED_all,
                                                              dPI.lag     = dPI.lag, 
                                                              dED_0.1.lag = dED_0.1.lag, 
                                                              dED_0.2.lag = dED_0.2.lag, 
                                                              dED_all.lag = dED_all.lag,
                                                              assort_0.1  = assortativity(g_0.1, types1 = rowSums(pcor.matrix > 0.1) - 1, directed = F),
                                                              assort_0.2  = assortativity(g_0.2, types1 = rowSums(pcor.matrix > 0.2) - 1, directed = F));
  }
  rm(speed, i, tau, 
     pcor.matrix, pcor.matrix.lag, pcor.matrix.lead, 
     dED_0.1, dED_0.2, dED_all, dPI,
     dED_0.1.lag, dED_0.2.lag,dED_all.lag, dPI.lag,
     g_0.1, g_0.2)
}

# Adding the network objects to the main data table (excluding markets and subsamples where no networks objects are available)
subsampleStats = merge(subsampleStats, do.call(rbind,networkStats), by = c("subsample","mkt"));

# Exporting the data table
save(list=c("subsampleStats"),file=paste0("output/subsampleStats"));

# Creating adjacency matrices for network plots
networks  = vector("list",length=length(eligible));
for(w in 1:length(eligible)){
  i = eligible[w];
  PI = matrix(ncol=N, nrow=K+1, dimnames=list(0:K,nlab));
  PI[,vecm[[i]]$nlab] = vecm[[i]]$PI[as.character(0:K),];
  networks[[w]] = data.table(subsample = i, eventTime=0:K, vecm[[i]]$pcorr[as.character(0:K),], vecm[[i]]$corr[as.character(0:K),], PI=PI);
}
networks = do.call(rbind,networks);
#networks = merge(networks, subsampleStats[mkt=="EBS",c("subsample","week","newsMatched"),with=F], all.x = TRUE, all.y = F, by="subsample")

# Exporting the adjacency matrices
save(list=c("networks","nlab"),file=paste0("output/networks_data_J",J,"_K",K));
