# This script provides a working example for estimating the VECM and obtaining network characteristics

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/build/");

# Loading functions required for the VECM estimation
source("code/VECM functions.R");

# Loading example data set
load(file = "input/vecm_pseudoinputdata");
N = length(quotes);

# VECM estimation inputs
J    = 10000; # Number of iterations to determine convergence of IRF and to obtain information shares
K    = 100;   # Maximum horizon for obtaining partial correlation network and distance
L    = NA;    # VECM lag length; when NA a lag selection criterion is used

# Retrieving price series on the chosen sampling frequency
p = VECM.freq(firstSec=37800,lastSec=38100,dataFreqMs=100,dateCurrent=19990101);

# Estimating the VECM
vecm = estimateVECM(fileName="temp/VECMdetails",p=p, L=L, J=J, K=K, nlab = nlab);

# Estimated adjacency matrix for event time tau
tau = "5";
round(matrix(vecm$pcorr[tau,],N,N,dimnames=list(nlab,nlab)),2);

# Price inefficiency
vecm$PI[tau,];

# Information shares
round(vecm$ISl,2);
round(vecm$ISu,2);
